#!/usr/bin/python3

'''
Copyright 2018 Gabriel Martins Trettel

This Source Code Form is subject to the terms of the Mozilla Public License,
v. 2.0. If a copy of the MPL was not distributed with this file, You can
obtain one at http://mozilla.org/MPL/2.0/.

This Source Code Form is “Incompatible With Secondary Licenses”, as defined
by the Mozilla Public License, v. 2.0.
'''


class Node():
    _used_node_attr = set()
    _header = { 'name':'VARCHAR',
                'label':'VARCHAR',
                'class':'VARCHAR',
                'visible':'BOOLEAN',
                'labelvisible':'BOOLEAN',
                'height':'DOUBLE',
                'x':'DOUBLE',
                'y':'DOUBLE',
                'width':'DOUBLE',
                'color':'VARCHAR' }

    def __init__(self, attr_dict):
        if self.valid(attr_dict):
            self.__attributes = {key:value for key,value in attr_dict.items()}
            Node._used_node_attr.update(attr_dict.keys())

    def valid(self, attr_dict):
        for attribute in attr_dict.keys():
            if attribute not in Node._header:
                raise ValueError('Trying to add a node with unidentified attributes')
        return True

    def __eq__(self, other):
        if isinstance(other, Node):
            return self.__attributes['name'] == other.__attributes['name']
        return self.__attributes['name'] == other

    def __hash__(self):
        return hash(self.__attributes['name'])

    def __str__(self):
        formatter = lambda key: str(self.__attributes[key]) if key in self.__attributes else ""
        return ",".join(formatter(key) for key in Node._header.keys() if key in Node._used_node_attr)

    @classmethod
    def update(cls, new_attr):
        Node._header.update(new_attr)

    @classmethod
    def header(cls):
        return {key:value for key,value in Node._header.items() if key in Node._used_node_attr}



# python3 setup.py sdist bdist_wheel
# twine upload --repository-url https://test.pypi.org/legacy/ dist/*

import setuptools

with open("READMEPYPI.md", "r") as fh:
    long_description = fh.read()

version = "1.0.1"

setuptools.setup(
    name="gdf_formatter",
    version=version,
    author="Gabriel Martins Trettel",
    author_email="gabrielmtrettel@gmail.com",
    description="GDF formatter for networks representation",
    long_description=long_description,
    url="https://github.com/GabrielTrettel/GDF_Formatter",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
        "Natural Language :: English",
    ],
)



#!/usr/bin/python3

'''
Copyright 2018 Gabriel Martins Trettel

This Source Code Form is subject to the terms of the Mozilla Public License,
v. 2.0. If a copy of the MPL was not distributed with this file, You can
obtain one at http://mozilla.org/MPL/2.0/.

This Source Code Form is “Incompatible With Secondary Licenses”, as defined
by the Mozilla Public License, v. 2.0.
'''

from .node import Node
from .edge import Edge


class Graph():
    def __init__(self, custom_nodes=dict(), custom_edges=dict(), allow_equal_edges=False, **kwargs):
        self.__allow_equal_edges = allow_equal_edges
        self.__node_list = set()
        self.__edge_list = set()
        Node.update(custom_nodes)
        Edge.update(custom_edges)
        Node.update(kwargs)

    def validateNode(self, new_node):
        if new_node in self.__node_list:
            raise ValueError('Trying to add two nodes with the same name')
        return True

    def addNode(self, **kwargs):
        new_node = Node(kwargs)
        if self.validateNode(new_node):
            self.__node_list.add(new_node)

    def validateLink(self, edge):
        if (edge.node1() not in self.__node_list) or (edge.node2() not in self.__node_list):
            raise ValueError('Trying to link non existent node')

        if not self.__allow_equal_edges:
            if edge in self.__edge_list:
                raise ValueError('Trying to create existent edge')

        return True

    def addLink(self, **kwargs):
        new_edge = Edge(kwargs)
        if self.validateLink(new_edge):
            self.__edge_list.add(new_edge)

    def dump(self, output_file=""):
        with open(output_file, 'w') as fl:
            f  = lambda k,v: "{} {}".format(k, v)

            node_header = "nodedef>" + ",".join( f(k,v) for k,v in Node.header().items() )
            fl.writelines(node_header)

            nodes = '\n' + "\n".join(str(node) for node in self.__node_list) + '\n'
            fl.writelines(nodes)

            edge_header = "edgedef>" + ",".join(f(k,v) for k,v in Edge.header().items() )
            fl.writelines(edge_header)

            edges = '\n' + "\n".join(str(edge) for edge in self.__edge_list)
            fl.writelines(edges)



#!/usr/bin/python3

'''
Copyright 2018 Gabriel Martins Trettel

This Source Code Form is subject to the terms of the Mozilla Public License,
v. 2.0. If a copy of the MPL was not distributed with this file, You can
obtain one at http://mozilla.org/MPL/2.0/.

This Source Code Form is “Incompatible With Secondary Licenses”, as defined
by the Mozilla Public License, v. 2.0.
'''

class Edge():
    _used_edge_attr = set()
    _header = { 'node1':'VARCHAR',
                'node2':'VARCHAR',
                'weight':'DOUBLE',
                'directed':'BOOLEAN',
                'color':'VARCHAR' }

    def __init__(self, attr_dict):
        if self.valid(attr_dict):
            self.__attributes = {key:value for key,value in attr_dict.items()}
            Edge._used_edge_attr.update(attr_dict.keys())

    def valid(self, attr_dict):
        for attribute in attr_dict.keys():
            if attribute not in Edge._header:
                raise ValueError('Trying to add a edge with unidentified attributes')
        return True

    def __eq__(self, node):
        return (self.__attributes['node1'] != node.__attributes['node1']) and (self.__attributes['node2'] != node.__attributes['node1'])

    def __hash__(self):
        return hash("".join(str(v) for v in self.__attributes.values()))

    def __str__(self):
        formatter = lambda key: str(self.__attributes[key]) if key in self.__attributes else ""
        return ",".join(formatter(key) for key in Edge._header.keys() if key in Edge._used_edge_attr)

    @classmethod
    def update(cls, new_attr):
        Edge._header.update(new_attr)

    @classmethod
    def header(cls):
        return {key:value for key,value in Edge._header.items() if key in Edge._used_edge_attr}

    def node1(self):
        return self.__attributes['node1']

    def node2(self):
        return self.__attributes['node2']
